----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		ps2 - behave_ps2 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		Simple PS/2 controller and wishbone interface, it sends the
--							scan code of the keyboard via the wishbone bus to the ZPU
--
-- Dependencies:		softcore_pkg	 - softcore_pkg.vhd
--
-- Revision:			0.02
-- Revision 0.01 - File created
-- Revision 0.02 - Added functionality to send the keyboard scan code via the
--						 wishbone bus
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- Including dependencies
library work;
use work.softcore_pkg.all;



entity ps2 is
	port (clk			: in  std_logic; -- System Clock
		 reset			: in  std_logic;
		 
		 interrupt		: out std_logic;
		 
		 slave_in		: in  wb_master_out;
		 slave_out		: out wb_master_in;
		 
		 ps2clk			: inout std_logic;
		 DataIn			: inout std_logic
		 );
end entity ps2;



architecture behave_ps2 of ps2 is

	--data: start bit, 8 data bits, parity bit, stop bit = 11 bits
	signal data 	: STD_LOGIC_VECTOR (10 downto 0)			:= (others => '0');
	--OutByte: 8 data bits
	signal OutByte : STD_LOGIC_VECTOR (7 downto 0) 			:= (others => '0');
	
	

begin
	-- The slave acknowledge is set by the cyc signal
	slave_out.ack <= slave_in.cyc;
	
	-- The slave data to be sent via the wishbone bus to the ZPU are set with the 8 bit scan code of the keyboard
	-- But only if the last to bits of the address are "11"
	slave_out.data <= (others => '0') when (reset = '1') else
							"000000000000000000000000" & OutByte when (slave_in.cyc = '1' and slave_in.addr (3 downto 2) = "11") else
							(others => '0');

	
	-- Scan code receiving and interrupt control
	ps2:
	process (ps2clk, reset)
		variable Counter : STD_LOGIC_VECTOR (4 downto 0) 	:= (others => '0');
	begin
		if (ps2clk'event and ps2clk = '0') then
			if (reset = '1') then
				interrupt <= '0';
				OutByte <= (others => '0');
				Counter := (others => '0');
				data    <= (others => '0');
				
			else
				-- The data of the keyboard are received serial, so the bits are saved in the fifo data
				data (10 downto 1) <= data (9 downto 0);
				data (0) <= DataIn;
				
				interrupt <= '0';
				
				-- When 10 Bits have been received set the interrupt and send the 8 data bits (the scan code)
				-- to the ZPU
				if (Counter = 10) then
					OutByte   <= data (9 downto 2);
					Counter   := (others => '0');
					interrupt <= '1';
				else
					Counter := Counter + 1;
				end if;
			end if;
		end if;
	end process ps2;

end behave_ps2;
