----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		wb_sram - behave_wb_sram 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		SRAM wishbone interface, includes the SRAM memory controller
--
-- Dependencies:		softcore_pkg	 - softcore_pkg.vhd
--
-- Revision:			0.02
-- Revision 0.01 - File created
-- Revision 0.02 - Added functionality to control the SRAM via the wishbone bus
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Including dependencies
library work;
use work.softcore_pkg.all;

entity wb_sram is
	port (clk		: in std_logic;
		
			reset			: in std_logic;
		
			interrupt	: out std_logic;
			
			slave_in		: in  wb_master_out;
			slave_out	: out wb_master_in;
			
			-- onboard SRAM1 (ISSI 256Kx16 SRAM, 10ns)
			ad				: out std_logic_vector (17 downto 0); -- address bus (18 Bit)
			we_n			: out std_logic; -- write enable
			oe_n			: out std_logic; -- output enable
			dio_a			: inout std_logic_vector (15 downto 0); -- data bus (16 Bit)
			ce_a_n		: out std_logic; -- chip enable
			ub_a_n		: out std_logic; -- upper byte enable
			lb_a_n		: out std_logic -- lower byte enable
			);
end wb_sram;


architecture behave_wb_sram of wb_sram is
   constant ADDR_W: integer:=18;
   constant DATA_W: integer:=16;
	
   signal addr		: std_logic_vector (ADDR_W - 1 downto 0);
	
   signal data_f2s: std_logic_vector (DATA_W - 1 downto 0);
	signal data_s2f: std_logic_vector (DATA_W - 1 downto 0);
   signal mem		: std_logic;
	signal rw		: std_logic;
   signal data_reg: std_logic_vector (DATA_W - 1 downto 0);

begin
   -- Instantiate the memory controller
   ctrl_unit: entity work.MemoryController
		port map(
			clk			=> clk,
			reset			=> reset,
			mem			=>	mem,
			rw 			=>	rw,
			addr			=>	addr,
			data_f2s		=>	data_f2s,
			ready			=>	open,
			data_s2f_r	=>	data_s2f,
			data_s2f_ur	=>	open,
			ad				=>	ad,
			we_n			=>	we_n,
			oe_n			=>	oe_n,
			dio_a			=> dio_a,
			ce_a_n		=> ce_a_n,
			ub_a_n		=> ub_a_n,
			lb_a_n		=> lb_a_n
		);

	 
	-- Interrupt not used, tie it to 0
	interrupt <= '0';

	-- The slave acknowledge is set by the cyc signal
	slave_out.ack <= slave_in.cyc;
	
	-- The slave data to be sent via the wishbone bus to the ZPU are set with the read SRAM data
	-- Because the data width of the SRAM is only 16 bit the other 16 bit are tied to 0
	slave_out.data (31 downto DATA_W) <= (others => '0');
	
	process (reset, slave_in.cyc, slave_in.we, slave_in.addr)
	begin
		if (reset = '1') then
			slave_out.data <= (others => '0');
		else
			if (slave_in.cyc = '1' and slave_in.addr (1 downto 0) = "11") then
				-- When the first 2 bits of the address are "11" then reading of SRAM is enabled and the SRAM datas will be
				-- sent to the ZPU
				-- 00 - Address
				-- 01 - Data in
				-- 11 - Data out
				slave_out.data (DATA_W - 1 downto 0) <= data_s2f;
			else
				slave_out.data <= (others => '0');
			end if;
		end if;
	end process;
										
				
	
	-- When the data in address is selected, get datas for the SRAM
	-- (last to address bits are "01")
	data_reg <= (others => '0') when (reset = '1') else
					slave_in.data (DATA_W-1 downto 0) when (slave_in.cyc = '1' and slave_in.we = '1' and slave_in.addr (1 downto 0) = "01");
	-- When the SRAM_address address is selected, get datas for the SRAM
	-- (last to address bits are "00")
	addr 		<= (others => '0') when (reset = '1') else
					slave_in.data (ADDR_W-1 downto 0) when (slave_in.cyc = '1' and slave_in.we = '1' and slave_in.addr (1 downto 0) = "00");
 
	
	-- Process to read from or write to SRAM
	-- the last bit (bit 31) of the SRAM_address is used to set the read or write mode
	-- (SRAM_address has just 18 bits so the rest of the 32 bits is unused and can be used for the modes)
   process(slave_in.cyc, slave_in.we, slave_in.addr, data_reg, slave_in.data)
   begin
     data_f2s <= (others=>'0');
	  if (slave_in.cyc = '1' and slave_in.we = '1' and slave_in.addr(1 downto 0) = "00") then
		  -- To enable the chip mem is put to 1
        mem <= '1';
		  
		  -- Set read (bit 31 = 1) or write mode (bit 31 = 0)
        rw <= std_logic (slave_in.data(31));
		  
		  -- In write mode send data to the SRAM
		  if (slave_in.data(31) = '0') then
				data_f2s <= data_reg;
		  end if;
     else
        mem <= '0';
        rw <= '1';
      end if;
   end process;
end behave_wb_sram;