----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		vga_sync - behave_vga_sync 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		VGA controller
--
-- Dependencies:		no
--
-- Revision:			0.01
-- Revision 0.01 - File received from Dennis Trebbels, trebbelsd@gmx.de
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity vga_sync is
   Port (clk 	: in  STD_LOGIC;
			reset 	: in  STD_LOGIC;
			video_on : out  STD_LOGIC;
			p_tick 	: out  STD_LOGIC;
			pixel_x 	: out  STD_LOGIC_VECTOR (9 downto 0);
			pixel_y 	: out  STD_LOGIC_VECTOR (9 downto 0);
			
			hsync 	: out  STD_LOGIC;
			vsync 	: out  STD_LOGIC
			);
end vga_sync;

architecture behave_vga_sync of vga_sync is

	-- VGA 640-by-480 sync parameters
	constant HD: integer := 640; -- horizontal display area
	constant HF: integer := 16;  -- h, front porch
	constant HB: integer := 48;  -- h, back porch
	constant HR: integer := 96;  -- h, retrace
	constant VD: integer := 480; -- vertical display area
	constant VF: integer := 10;  -- v, front porch
	constant VB: integer := 33;  -- v, back porch
	constant VR: integer := 2;   -- v, retrace
	
	-- mod-2 counter
	signal mod2_reg, mod2_next: std_logic;
	
	-- sync counters
	signal h_count_reg, h_count_next: unsigned (9 downto 0);
	signal v_count_reg, v_count_next: unsigned (9 downto 0);
	
	-- outpu buffer
	signal h_sync_reg, v_sync_reg: std_logic;
	signal h_sync_next, v_sync_next: std_logic;
	
	-- status signal
	signal h_end, v_end, pixel_tick: std_logic;
	
begin
	-- registers
	process (clk, reset)
	begin
		if (reset = '1') then
			mod2_reg <= '0';
			h_count_reg <= (others => '0');
			v_count_reg <= (others => '0');
			h_sync_reg  <= '0';
			v_sync_reg  <= '0';
		elsif (clk'event and clk = '1') then
			mod2_reg    <= mod2_next;
			h_count_reg <= h_count_next;
			v_count_reg <= v_count_next;
			h_sync_reg  <= h_sync_next;
			v_sync_reg  <= v_sync_next;
		end if;
	end process;	
	
	-- mod-2 circuit to generate 25 MHz enable tick
	mod2_next <= not mod2_reg;
	
	-- 25 MHz pixel tick
	pixel_tick <= '1' when mod2_reg = '1' else '0';
	
	-- status
	h_end <= -- end of horizontal counter
		'1' when h_count_reg = (HD + HF + HB + HR - 1) else -- 799
		'0';
	v_end <= -- end of horizontal counter
		'1' when v_count_reg = (VD + VF + VB + VR - 1) else -- 524
		'0';
		
	-- mod-800 horizontal sync counter
	process (h_count_reg, h_end, pixel_tick)
	begin
		if (pixel_tick = '1') then -- 25 MHz tick
			if (h_end = '1') then
				h_count_next <= (others => '0');
			else
				h_count_next <= h_count_reg + 1;
			end if;
		else
			h_count_next <= h_count_reg;
		end if;
	end process;
	
	-- mod-525 vertical sync counter
	process (v_count_reg, v_end, h_end, pixel_tick)
	begin
		if (pixel_tick = '1' and h_end = '1') then -- 25 MHz tick
			if (v_end = '1') then
				v_count_next <= (others => '0');
			else
				v_count_next <= v_count_reg + 1;
			end if;
		else
			v_count_next <= v_count_reg;
		end if;
	end process;
	
	-- horizontal and vertical sync, buffered to avoid glitch
	h_sync_next <=
		'1' when (h_count_reg >= (HD + HF))						-- 656
			and (h_count_reg <= (HD + HF + HR - 1)) else 	-- 751
		'0';
	v_sync_next <=
		'1' when (v_count_reg >= (VD + VF))						-- 490
			and (v_count_reg <= (VD + VF + VR - 1)) else		-- 491
		'0';
		
	-- video on/off
	video_on <=
		'1' when (h_count_reg < HD) and (v_count_reg < VD) else
		'0';
		
	--output signal
	hsync 	<= h_sync_reg;
	vsync 	<= v_sync_reg;
	pixel_x 	<= std_logic_vector (h_count_reg);
	pixel_y	<= std_logic_vector (v_count_reg);
	p_tick 	<= pixel_tick;	

end behave_vga_sync;
