----------------------------------------------------------------------------------
-- Company:				DHBW Mannheim, Germany
-- Engineer:			Johannes Steudle
-- 
-- Create Date:		11:00:00 02/18/2011 
-- Design Name:		ZPU
-- Module Name:		wb_vga - behave_wb_vga 
-- Project Name:		Softcore Implementation
-- Target Devices:	Xilinx Spartan 3 - FPGA XC3S200
-- Tool versions:		Xilinx ISE 10.1 WebPack
--
-- Description:		VGA wishbone interface, includes the VGA controller and FONT
--							ROM
--
-- Dependencies:		softcore_pkg	 - softcore_pkg.vhd
--
-- Revision:			0.02
-- Revision 0.01 - File created
-- Revision 0.02 - Added functionality to control the VGA port via the
--						 wishbone bus
--
-- Additional Comments: 
--	*  Project uses opensource softcore ZPU, Copyright 2004-2008 oharboe
--	   yvind Harboe - oyvind.harboe@zylin.com
-- *  The Basics of the implementation of the ZPU in to a wishbone system comes
--    from Matthias Meier - matthias2.meier@tu-dortmund.de
----------------------------------------------------------------------------------


-- Including libraries
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


-- Including dependencies
library work;
use work.softcore_pkg.all;



entity wb_vga is
	port (
		clk			: in std_logic;
		
		reset			: in std_logic;
		
		interrupt	: out std_logic;
		
		slave_in		: in  wb_master_out;
		slave_out	: out wb_master_in;
		
		hsync			: out std_logic;
		vsync			: out std_logic;
		rgb			: out std_logic_vector (2 downto 0)
	);
end wb_vga;


architecture behave_wb_vga of wb_vga is
	signal pixel_x		: std_logic_vector (9 downto 0);
	signal pixel_y		: std_logic_vector (9 downto 0);
	
   signal video_on	: std_logic;
	
	signal pixel_tick	: std_logic;
	
   signal rgb_reg		: std_logic_vector (2 downto 0);
	signal rgb_next	: std_logic_vector (2 downto 0);
	
	signal tmp 			: std_logic := '0';
	
begin

	-- instantiate VGA sync circuit
	vga_sync_unit: entity work.vga_sync
      port map(
			clk		=> clk,
			reset		=> reset,
			hsync		=> hsync,
         vsync		=> vsync,
			video_on	=> video_on,
         pixel_x	=> pixel_x,
			pixel_y	=> pixel_y,
         p_tick	=> pixel_tick
		);
					
	-- instantiate font ROM - used to write letters on a screen (80x30 field)
   font_gen_unit: entity work.font_test_gen
      port map(
			clk		=> clk,
			video_on	=> video_on,
			pixel_x	=> pixel_x,
			pixel_y	=> pixel_y,
			rgb_text	=> rgb_next
		);
					
	
	-- Interrupt not used, tie it to 0
	interrupt <= '0';

	
	-- The slave acknowledge is set by the cyc signal
	slave_out.ack <= slave_in.cyc;
	
	
	-- Wishbone output data to the ZPU not used, tie it to 0
	slave_out.data <= (others => '0');
	
	
					  
	-- rgb buffer
	process (clk, reset, slave_in.cyc, slave_in.we)
	begin
		if (reset = '1') then
		elsif (clk'event and clk = '1' and slave_in.cyc = '1' and slave_in.we = '1') then
			-- the first 3 bits are used to set the rgb color
			-- the 4th bit is used to change between to vga modes (see process below)
			if (slave_in.data (3) = '0') then
				tmp <= '1';
			else
				tmp <= '0';
			end if;
		end if;
	end process;
	
	-- process changes between two vga modes
	process (tmp, slave_in.cyc, slave_in.we)
	begin		
		if (tmp = '1') then
			-- if the 4th data bit is 0 then the first 3 bits of the datas are used to set the rgb color
			if (slave_in.cyc = '1' and slave_in.we = '1') then
				rgb_reg <= slave_in.data (2 downto 0);
			end if;
		else
			-- if the 4th data bit is 1 then the FONT ROM is used to display a certain text
			rgb_reg <= rgb_next;
		end if;		
	end process;
	
	-- Set 3 bit rgb color when video is on
	rgb <= rgb_reg when video_on = '1' else "000";
end behave_wb_vga;

