------------------------------------------------------------------------------
----                                                                      ----
----  Serveral parts of the uart.vhd originate from the ZPU Phi I/O:      ----
----                                                                      ----
----                                                                      ----
----                                                                      ----
----  ZPU Phi I/O                                                         ----
----                                                                      ----
----  http://www.opencores.org/                                           ----
----                                                                      ----
----  Description:                                                        ----
----  ZPU is a 32 bits small stack cpu. This is the minimum I/O devices   ----
----  assumed by the libc. They are a timer and an UART.@p                ----
----  Important! this is currently a simulation only model, no UART       ----
----  provided and it unconditionally generates a log.                    ----
----  Important! not all peripherals implemented!                         ----
----  Important! The enable signals assumes this is mapped @ 0x80A00xx.   ----
----                                                                      ----
----  To Do:                                                              ----
----  -                                                                   ----
----                                                                      ----
----  Author:                                                             ----
----    - Øyvind Harboe, oyvind.harboe zylin.com                          ----
----    - Salvador E. Tropea, salvador inti.gob.ar                        ----
----                                                                      ----
------------------------------------------------------------------------------
----                                                                      ----
---- Copyright (c) 2008 Øyvind Harboe <oyvind.harboe zylin.com>           ----
---- Copyright (c) 2008 Salvador E. Tropea <salvador inti.gob.ar>         ----
---- Copyright (c) 2008 Instituto Nacional de Tecnología Industrial       ----
----                                                                      ----
---- Distributed under the BSD license                                    ----
----                                                                      ----
------------------------------------------------------------------------------
----                                                                      ----
---- Design unit:      ZPUPhiIO(Behave) (Entity and architecture)         ----
---- File name:        phi_io.vhdl                                        ----
---- Note:             None                                               ----
---- Limitations:      Only for simulation.                               ----
---- Errors:           None known                                         ----
---- Library:          zpu                                                ----
---- Dependencies:     IEEE.std_logic_1164                                ----
----                   IEEE.numeric_std                                   ----
----                   std.textio                                         ----
----                   zpu.zpupkg                                         ----
----                   zpu.txt_util                                       ----
---- Target FPGA:      Spartan 3 (XC3S1500-4-FG456)                       ----
---- Language:         VHDL                                               ----
---- Wishbone:         No                                                 ----
---- Synthesis tools:  N/A                                                ----
---- Simulation tools: GHDL [Sokcho edition] (0.2x)                       ----
---- Text editor:      SETEdit 0.5.x                                      ----
----                                                                      ----
------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.mpsoc_config.all;
use work.mpsoc_pkg.all;
use work.uart_pkg.all;



entity uart is
	generic(brdivisor	: integer := 0);
	port(clk			: in  std_logic;															-- System Clock
		 reset			: in  std_logic;
		 interrupt		: out std_logic;															-- UART Rx Interrupt
		 uart_rx		: in  std_logic;															-- UART Rx input
		 uart_tx		: out std_logic;															-- UART Tx output
		 slave_in		: in  wb_master_out;
		 slave_out		: out wb_master_in);
end entity uart;



architecture behave_uart of uart is

	constant LOW_BITS : unsigned(1 downto 0) := (others => '0');
	constant TX_FULL  : std_logic := '0';
	constant RX_EMPTY : std_logic := '1';

	-- Rx
	signal rx_br		: std_logic;																-- Rx timing
	signal uart_read 	: std_logic;																-- ZPU read the value
	signal rx_avail		: std_logic;																-- Rx data available
	signal rx_data		: std_logic_vector(7 downto 0);											-- Rx data
	-- Tx
	signal tx_br		: std_logic;																-- Tx timing
	signal uart_write	: std_logic;																-- ZPU is writing
	signal tx_busy		: std_logic;																-- Tx can't get a new value


begin

	-- Rx
	rx: rxunit
		port map(clk => clk, reset => reset, enable => rx_br, read => uart_read, rxd => uart_rx, rxav => rx_avail, dataOut => rx_data);

	uart_read <= '1' when (slave_in.cyc = '1' and slave_in.we = '0' and slave_in.addr(3 downto 2) = "01") else '0';

	-- Tx
	tx: txunit
		port map(clk => clk, reset => reset, enable => tx_br, load => uart_write, txd => uart_tx, busy => tx_busy, dataIn => slave_in.data(7 downto 0));

	uart_write <= '1' when (slave_in.cyc = '1' and slave_in.we = '1' and slave_in.addr(3 downto 2) = "10") else '0';

	-- Rx timing
	rx_timer: brgen
		generic map(count => brdivisor)
		port map(clk => clk, reset => reset, ce => '1', output => rx_br);

	-- Tx timing
	tx_timer: brgen																					-- 4 Divider for Tx
		generic map(count => 4)
		port map(clk => clk, reset => reset, ce => rx_br, output => tx_br);


	slave_out.ack <= slave_in.cyc;
	slave_out.data <= "00000000000000000000000" & rx_avail & rx_data when (slave_in.cyc = '1' and slave_in.addr(3 downto 2) = "01") else
					  "00000000000000000000000" & not(tx_busy) & "00000000" when (slave_in.cyc = '1' and slave_in.addr(3 downto 2) = "10") else
					  (others => '0');


	uart:
	process(clk, reset)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				interrupt <= '0';
			else
				if (rx_avail = '1') then
					interrupt <= '1';
				end if;

				if (slave_in.cyc = '1' and slave_in.we = '0' and slave_in.addr(3 downto 2) = "01") then
						interrupt <= '0';
				end if;
			end if;
		end if;
	end process uart;
end behave_uart;
